package cz.cuni.mff.emoticonposhbot_c;

import java.io.IOException;
import java.util.logging.Level;

import cz.cuni.amis.pogamut.base.agent.impl.AgentId;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.emohawk.agent.module.sensomotoric.CharacterType;
import cz.cuni.amis.pogamut.sposh.ut2004.StateSposhLogicController;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004BotController;
import cz.cuni.amis.pogamut.ut2004.bot.params.UT2004BotParameters;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Initialize;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.ConfigChange;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.GameInfo;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.InitedMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Self;
import cz.cuni.amis.pogamut.ut2004.communication.parser.UT2004Parser;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004BotRunner;
import cz.cuni.amis.utils.exception.PogamutException;

/**
 * Create simple bot that is running around the map and expresses random thoughts.
 * 
 * @author Jimmy
 */
@SuppressWarnings("rawtypes")
public class EmoticonBotLogic extends StateSposhLogicController<UT2004Bot, EmoticonBotContext>
{

    private String SPOSH_PLAN_RESOURCE = "sposh/plan/BotPlan.lap";

    @Override
    protected String getPlan() throws IOException
    {
        return getPlanFromResource(SPOSH_PLAN_RESOURCE);
    }

    /**
     * Create context that can be accessed in every state primitive.
     * 
     * @return new context of this bot.
     */
    @Override
    protected EmoticonBotContext createContext()
    {
        EmoticonBotContext result = new EmoticonBotContext(bot);
        result.getLog().setLevel(Level.WARNING);
        return result;
    }

    /**
     * Handshake with GameBots2004 is over - bot has information about the map in its world view.
     * Many agent modules are usable since this method is called.
     * 
     * @param gameInfo information about the game type
     * @param currentConfig information about configuration
     * @param init information about configuration
     */
    @Override
    public void botInitialized(GameInfo gameInfo, ConfigChange currentConfig, InitedMessage init)
    {
        ((UT2004Parser) bot.getEventBus().getComponent(UT2004Parser.COMPONENT_ID)).getLog().setLevel(Level.OFF);
        bot.getLogger().getCategory("Yylex").setLevel(Level.OFF);
    }

    /**
     * Here we can modify initializing command for our bot, e.g., sets its name or skin.
     * 
     * @return instance of {@link Initialize}
     */
    @Override
    public Initialize getInitializeCommand()
    {
        Initialize result = new Initialize().setName(this.getBot().getParams().getAgentId().getName().getFlag());
        if (result.getName().equals("EmotionOne")) {
            result.setClassName(CharacterType.NATALIE.getUE2Class());
            result.setLocation(new Location(-800, -8000, -1000));
        } else {
            result.setClassName(CharacterType.THOMAS.getUE2Class());
            result.setLocation(new Location(-800, -7600, -1000));
        }
        return result;
    }

    @Override
    public void botFirstSpawn(GameInfo gameInfo, ConfigChange currentConfig, InitedMessage init, Self self) {
        super.botFirstSpawn(gameInfo, currentConfig, init, self);
        
        bot.getLogger().getCategory(SPOSH_LOG_CATEGORY).setLevel(Level.WARNING);
        bot.getLogger().getCategory(UT2004BotController.USER_LOG_CATEGORY_ID).setLevel(Level.WARNING);                
    }     
    
    /**
     * Create an ExternalBot with custom made logic and
     * try to connect to Unreal Server at localhost:3000
     * 
     * @param args the command line arguments
     * @throws PogamutException
     */
    @SuppressWarnings("unchecked")
    public static void main(String[] args) throws PogamutException
    {
        new UT2004BotRunner(EmoticonBotLogic.class, "SPOSH-EmotionBot")
                .setMain(true)
                .setLogLevel(Level.WARNING)
                .startAgents(
                        new UT2004BotParameters[] { new UT2004BotParameters().setAgentId(new AgentId("EmotionOne")),
                                new UT2004BotParameters().setAgentId(new AgentId("EmotionTwo")) });
    }
}
